
# $Id: dp-model.R,v 1.16 2005/12/08 18:30:32 jon Exp $
# $Source: /projects/starl/cvs/dp-eb/src/dp-model.R,v $

###
### Copyright (c) 2004, 2005, David M. Blei and Jon D. McAuliffe.
### This software is distributed under the terms of the GNU
### General Public License.
###

stopifnot(Sys.getenv("DPEB_LIB") != "");

source(paste(Sys.getenv("DPEB_LIB"), "/dp-gibbs.R", sep = ""));
source(paste(Sys.getenv("DPEB_LIB"), "/dp-eb-estimates.R", sep = ""));
source(paste(Sys.getenv("DPEB_LIB"), "/rdp.R", sep = ""));
source(paste(Sys.getenv("DPEB_LIB"), "/utils.R", sep = ""));
source(paste(Sys.getenv("DPEB_LIB"), "/dp-conj-gibbs.R", sep = ""));


simulate.data <- function()
{
  dp      <<- new.dp(rg0, alpha);
  thetas  <<- rdp(N, dp);
  dataset <<- rdata(N, thetas);
}


save.round <- function(round.num, dir)
{
  filename <- sprintf("%s/round=%03d.Rdata", dir, as.integer(round.num));
  save(gsamples, gibbs.summary, eb.results, file = filename);
}


# burn.in is measured in multiples of lag.

simulation <- function(n, m, b, lag, burn.in, num.rounds,
                       alpha, rg0, dg0, rdata, ddata,
                       seed, sim.dir)
{
  stopifnot(system(paste("mkdir ", sim.dir, sep = "")) == 0);

  set.seed(seed);
  seed <<- seed;
  EPS  <<- 1e-6;

  N       <<- n;
  M       <<- m;
  B       <<- b;
  LAG     <<- lag;
  BURN.IN <<- burn.in;

  alpha <<- alpha;
  rg0   <<- rg0;
  dg0   <<- dg0;
  rdata <<- rdata;
  ddata <<- ddata;

  gibbs.summary <<- array(dim = c(B, LAG, 2));

  simulate.data();

  # fit a kde to the true theta stars.
  dut <<- density(unique(thetas), bw = "SJ-dpi", kernel = "gaussian",
                  n = 1024);
  x.grid <- dut$x;
  grid.step <- x.grid[2] - x.grid[1];
  y.grid <- dut$y;
  dg0.hat.ideal <<- function(x)
  {
    res <- numeric(length(x));
    good <- ( (x >= x.grid[1]) & (x < x.grid[length(x.grid)]) );
    res[!good] <- 0;

    if (!any(good))
    {
      return(res);
    }

    left.bdys <- sapply(x[good], function(v) max(which(v >= x.grid)));
    lambdas <- (x[good] - x.grid[left.bdys]) / grid.step;
    res[good] <-
      (1 - lambdas) * y.grid[left.bdys] + lambdas * y.grid[left.bdys + 1];
    res;
  }
  attr(dg0.hat.ideal, "x.grid") <<- x.grid;
  attr(dg0.hat.ideal, "y.grid") <<- y.grid;

  save(N, M, B, LAG, BURN.IN, alpha, rg0, dg0, rdata, ddata,
       dp, thetas, dataset, seed, dut, dg0.hat.ideal,
       file = paste(sim.dir, "/global-state.Rdata", sep = ""));

  # to get the initial EB estimates, use a single gibbs sample.
  gsamples <<- list(list(classes     = numeric(N),
                         theta.stars = numeric(N),
                         counts      = numeric(N)));

  first.state(1);
  eb.results  <<- dp.eb.estimates(first = TRUE);
  save.round(0, sim.dir);

  gsamples <<-
    lapply(1:B, function(i) list(classes     = numeric(N),
                                 theta.stars = numeric(N),
                                 counts      = numeric(N)));

  for (i in 1:num.rounds)
  {
    cat("round = ", i, "...\n", sep = "");

    rg0.hat    <<- eb.results$rg0.hat;
    alpha.hat  <<- eb.results$alpha.hat;

    first.state(B);
    collect.B.samples();

    eb.results <<- dp.eb.estimates();

    save.round(i, sim.dir);
  }
}


dp.eb.fit.model <- function(dataset, ddata, alpha.init, m, b, lag, burn.in,
                            num.rounds,
                            should.do.data.sd.eb, seed, output.dir)
{
  stopifnot(system(paste("mkdir -p ", output.dir, sep = "")) == 0);

  set.seed(seed);
  seed <<- seed;
  EPS  <<- 1e-6;

  dataset <<- dataset;
  N       <<- length(dataset);
  ddata   <<- ddata;

  ALPHA.INIT <<- alpha.init;

  M       <<- m;
  B       <<- b;
  LAG     <<- lag;
  BURN.IN <<- burn.in;

  gibbs.summary <<- array(dim = c(B, LAG, 2));

  save(ALPHA.INIT, M, B, LAG, BURN.IN, dataset, N, ddata, seed,
       file = paste(output.dir, "/global-state.Rdata", sep = ""));

  # to get the initial EB estimates, use a single gibbs sample.
  gsamples <<- list(list(classes     = numeric(N),
                         theta.stars = numeric(N),
                         counts      = numeric(N)));

  first.state(1);
  eb.results  <<- dp.eb.estimates(first = TRUE);
  if (should.do.data.sd.eb)
  {
    eb.results$data.sd.hat <<- data.sd.m.step(first = TRUE);
  }

  save.round(0, output.dir);

  if (num.rounds == 0)
  {
    return();
  }

  gsamples <<-
    lapply(1:B, function(i)
           list(classes     = numeric(N),
                theta.stars = numeric(N),
                counts      = numeric(N)));

  for (i in 1:num.rounds)
  {
    cat("round = ", i, "...\n", sep = "");

    rg0.hat    <<- eb.results$rg0.hat;
    alpha.hat  <<- eb.results$alpha.hat;
    if (should.do.data.sd.eb)
    {
      data.sd.hat <<- eb.results$data.sd.hat;
    }

    first.state(B);
    collect.B.samples();

    eb.results <<- dp.eb.estimates();
    if (should.do.data.sd.eb)
    {
      eb.results$data.sd.hat <<- data.sd.m.step();
    }

    save.round(i, output.dir);
  }
}
